/*
  UNDER CONSTRUCTION ...
  This sketch uses an Arduino Nano with NOKIA 5110display
  It is also a FM receiver capable to tune your local FM stations.

  Read more on https://pu2clr.github.io/RDA5807/

  Wire up on Arduino UNO, Nano or Pro mini

  | Device name               | Device Pin / Description  |  Arduino Pin  |
  | --------------------------| --------------------      | ------------  |
  | NOKIA 5110                |                           |               |
  | --------------------------| ------------------------- | ------------  |
  |                           | (1) RST (RESET)           |     8         |
  |                           | (2) CE or CS              |     9         |
  |                           | (3) DC or DO              |    10         |
  |                           | (4) DIN or DI or MOSI     |    11         |
  |                           | (5) CLK                   |    13         |
  |                           | (6) VCC  (3V-5V)          |    +VCC       |
  |                           | (7) BL/DL/LIGHT           |    +VCC       |
  |                           | (8) GND                   |    GND        |
  | --------------------------| ------------------------- | --------------|
  | RDA5807                   |                           |               | 
  |                           | ------------------------- | --------------|
  |                           | SDIO (pin 8)              |     A4        |
  |                           | SCLK (pin 7)              |     A5        |
  | --------------------------| --------------------------| --------------|
  | Buttons                   |                           |               |
  |                           | Volume Up                 |      4        |
  |                           | Volume Down               |      5        |
  |                           | Stereo/Mono               |      6        |
  |                           | RDS ON/off                |      7        |
  |                           | SEEK (encoder button)     |     12        |
  | --------------------------| --------------------------|---------------| 
  | Encoder                   |                           |               |
  |                           | --------------------------|---------------| 
  |                           | A                         |       2       |
  |                           | B                         |       3       |

  About 77% of the space occupied by this sketch is due to the library for the TFT // display.


  Prototype documentation: https://pu2clr.github.io/RDA5807/
  PU2CLR RDA5807 API documentation: https://pu2clr.github.io/RDA5807/extras/apidoc/html/

  By PU2CLR, Ricardo,  Feb  2023.
*/

#include <RDA5807.h>

#include <Adafruit_GFX.h>     // Core graphics library
#include <Adafruit_PCD8544.h> // See: https://www.electronoobs.com/eng_arduino_Adafruit_PCD8544.php
#include <SPI.h>

#include "Rotary.h"

// NOKIA Display pin setup
#define NOKIA_RST  8  // RESET
#define NOKIA_CE   9  // Some NOKIA devices show CS
#define NOKIA_DC  10  // 
#define NOKIA_DIN 11  // MOSI
#define NOKIA_CLK 13  // SCK
#define NOKIA_LED  0  // 0 if wired to +3.3V directly


#define COLOR_BLACK 0x0000
#define COLOR_WHITE 0xFFFF

// Enconder PINs
#define ENCODER_PIN_A 2
#define ENCODER_PIN_B 3

// Buttons controllers
#define VOLUME_UP 4     // Volume Up
#define VOLUME_DOWN 5   // Volume Down
#define SWITCH_STEREO 6 // Select Mono or Stereo
#define SWITCH_RDS 7    // SDR ON or OFF
#define SEEK_FUNCTION 12

#define POLLING_TIME  2000
#define POLLING_RDS     80


bool bSt = true;
bool bRds = false;
bool bShow = false;
uint8_t seekDirection = 1; // 0 = Down; 1 = Up. This value is set by the last encoder direction.

long pollin_elapsed = millis();

int maxX1;
int maxY1;

// Encoder control variables
volatile int encoderCount = 0;
uint16_t currentFrequency;

// Encoder control
Rotary encoder = Rotary(ENCODER_PIN_A, ENCODER_PIN_B);

// Nokia 5110 display
Adafruit_PCD8544 display = Adafruit_PCD8544(NOKIA_DC, NOKIA_CE, NOKIA_RST);

RDA5807 rx;

void setup()
{

  pinMode(ENCODER_PIN_A, INPUT_PULLUP);
  pinMode(ENCODER_PIN_B, INPUT_PULLUP);

  // Push button pin
  pinMode(VOLUME_UP, INPUT_PULLUP);
  pinMode(VOLUME_DOWN, INPUT_PULLUP);
  pinMode(SWITCH_STEREO, INPUT_PULLUP);
  pinMode(SWITCH_RDS, INPUT_PULLUP);
  pinMode(SEEK_FUNCTION, INPUT_PULLUP);

  // Start the Nokia display device
  display.begin();  
  display.setContrast(20);    // You may need adjust this value for you Nokia 5110
  showSplash();
  showTemplate();

  // Encoder interrupt
  attachInterrupt(digitalPinToInterrupt(ENCODER_PIN_A), rotaryEncoder, CHANGE);
  attachInterrupt(digitalPinToInterrupt(ENCODER_PIN_B), rotaryEncoder, CHANGE);

  rx.setup();
  rx.setVolume(6);
  rx.setMono(false); // Force stereo
  // rx.setRBDS(true);  //  set RDS and RBDS. See setRDS.
  rx.setRDS(true);
  rx.setRdsFifo(true);

  rx.setFrequency(10390); // It is the frequency you want to select in MHz multiplied by 100.

  rx.setSeekThreshold(50); // Sets RSSI Seek Threshold (0 to 127)
  showStatus();
}


/*
    Reads encoder via interrupt
    Use Rotary.h and  Rotary.cpp implementation to process encoder via interrupt
*/
void rotaryEncoder()
{ // rotary encoder events
  uint8_t encoderStatus = encoder.process();
  if (encoderStatus)
    encoderCount = (encoderStatus == DIR_CW) ? 1 : -1;
}

/*
   Shows the static content on  display
*/
void showTemplate()
{

  maxX1 = display.width() - 2;
  maxY1 = display.height() - 2;

  // TO DO: The frame of the screen

}


/*
   Shows frequency information on Display
*/
void showFrequency()
{
  char freq[10];

  currentFrequency = rx.getFrequency();
  display.setTextSize(2);
  rx.convertToChar(currentFrequency,freq,5,3,',', true);
  display.setCursor(3, 10);
  display.print(freq);
  display.display();

}

void showFrequencySeek()
{
  display.clearDisplay();
  showFrequency();
  display.display();
}

/*
    Show some basic information on display
*/
void showStatus()
{
  display.clearDisplay();
  display.setTextColor(BLACK);
  showFrequency();
  showStereoMono();
  showRSSI();
  display.display();
}

/* *******************************
   Shows RSSI status
*/
void showRSSI()
{
  char rssi[12];
  display.setTextSize(1);
  rx.convertToChar(rx.getRssi(),rssi,3,0,'.');
  strcat(rssi,"dB");
  display.setCursor(53, 0);
  display.print(rssi);

}

void showStereoMono() {
  display.setTextSize(1);
  display.setCursor(0, 2);
  if (rx.isStereo() ) { 
    display.print("ST");
  } else {
    display.print("MO");
  }
}

/*********************************************************
   RDS
 *********************************************************/
char *rdsMsg;
char *stationName;
char *rdsTime;
long stationNameElapsed = millis();
long polling_rds = millis();
long clear_fifo = millis();

void showRDSMsg()
{
  rdsMsg[22] = '\0';   // Truncate the message to fit on // display.  You can try scrolling

  // TO DO: show RDS message   
  // display.setTextSize(1);
  // display.setCursor(0, 30);  
  // display.print(rdsMsg);
  // delay(100);
}

/**
   TODO: process RDS Dynamic PS or Scrolling PS
*/
void showRDSStation()
{

  // TO DO 
  // display.setTextSize(1);
  // display.setCursor(0, 40);  
  // display.print(stationName);
  // delay(100);
  
}

void showRDSTime()
{
  // TO DO
  // display.setCursor(0, 40);  
  // display.print(rdsTime);  
  // delay(100);
}


void clearRds() {
  bShow = false;
}

void checkRDS()
{
  // check if RDS currently synchronized; the information are A, B, C and D blocks; and no errors
  if ( rx.hasRdsInfo() ) {
    rdsMsg = rx.getRdsText2A();
    stationName = rx.getRdsText0A();
    rdsTime = rx.getRdsTime();
    if (rdsMsg != NULL)
      showRDSMsg();

    if ((millis() - stationNameElapsed) > 1000)
    {
      if (stationName != NULL)
        showRDSStation();
      stationNameElapsed = millis();
    }

    if (rdsTime != NULL)
      showRDSTime();
  }

  if ( (millis() - clear_fifo) > 10000 ) {
    rx.clearRdsFifo();
    clear_fifo = millis();
    
  }
  showStatus();
}

void showRds() {

  display.setCursor(25, 0);
  if ( bRds ) { 
    display.print("RDS");
  } else {
    display.print("   ");
  }  
  display.display();
  // TO DO
  // checkRDS();
  
}

/*********************************************************

 *********************************************************/

void showSplash()
{
  display.clearDisplay();
  display.display();
  display.setTextColor(BLACK);
  // Splash - Change it by the your introduction text.
  display.setCursor(0, 0);
  display.setTextSize(2);
  display.print("RDA5810");
  display.setCursor(0, 15);
  display.print("Arduino");
  display.setCursor(0, 30);
  display.print("Library");
  display.display();
  delay(2000);
  display.clearDisplay();
  delay(4000);
  display.display();  
}




void doStereo() {
  rx.setMono((bSt = !bSt));
  bShow =  true;
  showStereoMono();
  delay(100);
}

void doRds() {
  rx.setRDS((bRds = !bRds));
  showRds();
}

/**
   Process seek command.
   The seek direction is based on the last encoder direction rotation.
*/
void doSeek() {
  rx.seek(RDA_SEEK_WRAP, seekDirection, showFrequencySeek);  // showFrequency will be called by the seek function during the process.
  delay(200);
  bShow =  true;
  showStatus();
}

void loop()
{

  // Check if the encoder has moved.
  if (encoderCount != 0)
  {
    if (encoderCount == 1) {
      rx.setFrequencyUp();
      seekDirection = RDA_SEEK_UP;
    }
    else {
      rx.setFrequencyDown();
      seekDirection = RDA_SEEK_DOWN;
    }
    showStatus();
    bShow = true;
    encoderCount = 0;
  }

  if (digitalRead(VOLUME_UP) == LOW)
    rx.setVolumeUp();
  else if (digitalRead(VOLUME_DOWN) == LOW)
    rx.setVolumeDown();
  else if (digitalRead(SWITCH_STEREO) == LOW)
    doStereo();
  else if (digitalRead(SWITCH_RDS) == LOW)
    doRds();
  else if (digitalRead(SEEK_FUNCTION) == LOW)
    doSeek();

  if ( (millis() - pollin_elapsed) > POLLING_TIME ) {
    showStatus();
    if ( bShow ) clearRds();
    pollin_elapsed = millis();
  }

  if ( (millis() - polling_rds) > POLLING_RDS) {
    if ( bRds ) {
      showRds();
    }
    polling_rds = millis();
  }

  delay(50);
}
